{smcl}
{* *! version 1.0.0 dec2012}{...}
{cmd:help gbsslice} 
{hline}

{title:Title}

{p2colset 5 17 19 2}{...}
{p2col :{hi:gbsslice} {hline 2}}Slice sampler for use in Gibbs sampling{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:gbsslice}  {it:logpost rowvector parameter}
{ifin}
{weight}
[{cmd:,} {it:options}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} in
{helpb mhsnorm} for more details on how to write the program for the log
posterior.

{phang}
{it:rowvector} must be the name of the row vector containing the parameter
values.

{phang}
{it:parameter} must be the number of the parameter that is being updated.

{synoptset 12}{...}
{synopthdr}
{synoptline}
{synopt:{opt s:tepsize(#)}}size of steps for finding {cmd:gbsslice} edge{p_end}
{synopt:{opt n:step(#)}}maximum number of steps to find an edge{p_end}
{synopt:{opt l:bound(#)}}low bound on the parameter{p_end}
{synopt:{opt u:bound(#)}}upper bound on the parameter{p_end}
{synopt:{opt max:try(#)}}maximum number of attempts to sample new value{p_end}
{synopt:{opt logp(#)}}log posterior at the current point{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
{opt fweight}s, {opt aweight}s, {opt iweight}s, and {opt pweight}s are
allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:gbsslice} uses slice sampling to sample from a one-dimensional posterior
distribution.  It is useful in Gibbs sampling.  {cmd:gbsslice} is a Metropolis
algorithm that creates its proposal by choosing a random height above the
current point and then moving outward by adding steps until a slice is created
that has a greater width than the density.  Random points are generated within
the slice.  The slice is shrunk using rejected points.  Calculations are
performed by a Mata function, but it can be called from Stata.  It requires a
user-written program for calculating the log posterior that may be written in
Stata or Mata.  The Mata code is stored in the library {cmd:libmcmc}, which
must be installed for the command to work.


{title:Options}

{phang}
{opt stepsize(#)} specifies the size of the increments that are added to try
to locate the end of the slice. Choosing a value that is approximately
equal to the range of the conditional distribution usually works well. The
default is {cmd:stepsize(1)}.{p_end}

{phang}
{opt nstep(#)} specifies the maximum number of steps outward that are
attempted when trying to locate the end of the slice. The default is
{cmd:nstep(6)}.{p_end}

{phang}
{opt lbound(#)} specifies the lower bound on the parameter. The slice will not
extend below this value. The default is minus infinity.{p_end}

{phang}
{opt ubound(#)} specifies the upper bound on the parameter. The slice will not
extend above this value. The default is infinity.{p_end}

{phang}
{opt maxtry(#)} specifies the maximum number of attempts to generate a
parameter value before the algorithm gives up. The default is
{cmd:maxtry(50)}. Typically, slice sampling finds a new value within a handful
of attempts, so exceeding 50 would probably indicate a very poorly chosen
stepsize or an error in {it:logpost}.

{phang}
{opt logp(#)} specifies the value of the log posterior at the current point.
If a nonmissing value is given, that value is used; otherwise, the value is
calculated using the user's program {it:logpost}. Specifying the value when
it is known will save one call to the user's program and so speed the
algorithm.{p_end}


{title:Stored results}

{pstd}
{cmd:gbsslice} stores the following in {cmd:r()}:

{synoptset 12 tabbed}{...}
{p2col 5 12 16 2: Scalars}{p_end}
{synopt:{cmd:r(logp)}}log posterior at the new point{p_end}
{synopt:{cmd:r(try)}}number of evaluations of the log posterior needed by
the algorithm{p_end}
{p2colreset}{...}


{title:Example}

{phang}
{cmd:. gbsslice mylogpost b 3, stepsize(0.1)}{p_end}


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
