*============================================================================
* mcmcgeweke:  Form of the Geweke test that compares the start and end of an MCMC run
* Author:     John Thompson
* Date:       Dec 2012
*============================================================================

program mcmcgeweke, rclass byable(recall)
   version 12.1
   syntax varlist [if] [in] , [       ///
   Percentages(numlist min=2 max=2)   /// percent of chain at start and finish
   ]
   
   marksample touse , novarlist
   
   preserve
   qui keep if `touse'
*-----------------------------
* Count number of parameters
*-----------------------------
   local np : word count `varlist'
*------------------------------
* Set percentages for start
* and end of the chain
*------------------------------
   if "`percentages'" == "" {
      local p1 = 10
      local p2 = 50
   }
   else {
      local i = 0
      foreach v of numlist `percentages' {
         local ++i
         local p`i' = `v'
      }
      local sp = `p1' + `p2'
      if `sp' == . | `p1' < 0 | `p2' < 0 | `sp' > 100 {
         di as err "mcmcgeweke: error in percentages"
         exit(499)
      }
   }
   cap qui tsset
   local oldt = r(timevar)
   tempvar end t
   gen `t' = _n
   local limit1 = _N*`p1'/100
   local limit2 = _N-_N*`p2'/100
   gen `end' = `t' > `limit2'
   tempname b vb
   qui tsset `t'
*---------------------------
* Output Header
*---------------------------
   di as text _newline _dup(60) "-"
   di as text "Parameter" _col(12) "Statistic" _col(25) "First Section" _col(45) "Final Section"
   di as text _dup(60) "-"
   local i = 0
*---------------------------
* Test each variable
*---------------------------
   foreach v of varlist `varlist' {
      local i = `i'+1
      qui count if `t' <= `limit1'
      local n1 = r(N)
      qui prais `v' if `t' <= `limit1'
      matrix `b' = e(b)
      matrix `vb' = e(V)
      local m1 = `b'[1,1]
      local s1 = sqrt(`vb'[1,1])
      local rho1 = e(rho)
      qui count if `t' > `limit2'
      local n2 = r(N)
      qui prais `v' if `t' > `limit2'
      matrix `b' = e(b)
      matrix `vb' = e(V)
      local rho2 = e(rho)
      local m2 = `b'[1,1]
      local s2 = sqrt(`vb'[1,1])
*------------------------------------
* Display results
*------------------------------------
      if `i' == 1 {
         di as text _col(12) "size" _continue
         di as res _col(25) %4.1f `p1' "% (n = `n1')" _col(45) %4.1f `p2' "% (n=`n2')"
      }
      di as text _dup(60) "-"
      di as text "`v'" _continue
      di as text _col(12) "mean" as res _col(25) %10.4f `m1' _col(45) %10.4f `m2'
      di as text _col(12) "st error" as res _col(25) %10.4f `s1' _col(45) %10.4f `s2'
      di as text _col(12) "autocorr" as res _col(25) %10.4f `rho1' _col(45) %10.4f `rho2'
      local z = abs(`m2'-`m1')/sqrt(`s1'^2+`s2'^2)
      local p = 2*(1-normal(`z'))
      di as text _col(12) "mean diff" as res _col(35) %10.4f `m2'-`m1'
      di as text _col(12) "se diff" as res _col(35) %10.4f sqrt(`s1'^2+`s2'^2)
      di as text _col(12) "z" as res _col(35) %10.4f `z'
      di as text _col(12) "p-value" as res _col(35) %10.4f `p'
      return scalar m1_`i' = `m1'
      return scalar m2_`i' = `m2'
      return scalar se1_`i' = `s1'
      return scalar se2_`i' = `s2'
      return scalar z_`i' = `z'
      return scalar p_`i' = `p'
   }
   di as text _dup(60) "-"
   restore
   cap qui tsset `oldt'
end
