{smcl}
{* *! version 1.0.0  dec2012}{...}
{cmd:help mhsmnorm} 
{hline}

{title:Title}

{p2colset 5 17 19 2}{...}
{p2col :{hi:mhsmnorm} {hline 2}}Metropolis algorithm for block updating based
on a random walk with a multivariate Gaussian proposal distribution{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:mhsmnorm} {it:logpost rowvector parameter} {ifin} {weight}{cmd:,}
{opt c:holesky(name)}
[{opt logp(#)}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} in
{helpb mhsnorm} for more details on how to write the program for the log
posterior.

{phang}
{it:rowvector} must be the name of the row vector containing the parameter
values.

{phang}
{it:parameter} must be the number of the parameter that is being updated.

{synoptset 16 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opt c:holesky(name)}}Stata matrix containing the Cholesky decomposition of the variance matrix of the multivariate normal proposal distribution{p_end}
{synopt:{opt logp:(#)}}value of log posterior at current point{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:cholesky()} is required.{p_end}
{p 4 6 2}
{opt fweight}s are allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:mhsmnorm} uses the Metropolis algorithm to create a single update of a
block of parameters.  It uses a multivariate Gaussian proposal distribution
and is suitable for parameters that can take any real value, although it
generally works well for parameters with restricted ranges provided that their
posterior is well away from the bounds. Repeated calls to {cmd:mhsmnorm} can
be used to create an MCMC algorithm and hence estimate the posterior
distribution of the set of parameters. The user must supply a program that
evaluates the log posterior; this program must have the structure described in
{helpb mhsnorm}.  Parameters are passed in a row vector. The efficiency but not
the validity of the algorithm is affected by the choice of matrix given in the
{cmd:cholesky()} option. This matrix represents the Cholesky decomposition of
the variance-covariance matrix of the proposal distribution. It is not valid
to alter this matrix on the basis of previous simulations from the same
chain.{p_end}


{title:Options}

{phang}
{opt cholesky(name)} specifies the Cholesky decomposition of the
variance-covariance matrix of the multivariate normal (Gaussian) proposal
distribution.{p_end}

{phang}
{opt logp(#)} specifies the value of the log posterior at the current point.
If given and not missing, then this value is not recalculated, which saves one
call to the user's program {it:logpost}.{p_end}


{title:Stored results}

{pstd}
{cmd:mhsmnorm} stores the following in {cmd:r()}:

{p2colset 9 20 22 2}{...}
{p2col 5 12 16 2: Scalars}{p_end}
{synopt:{cmd:r(logp)}}log posterior under the updated set of parameters{p_end}
{synopt:{cmd:r(accept)}}{cmd:1} if the proposal was accepted and {cmd:0} otherwise{p_end}
{p2colreset}{...}


{title:Example}

{pstd}Block update parameters 3 and 4 using a proposal variance-covariance
matrix V{p_end}
{phang2}{cmd:. matrix V = (10,2\2,5)}{p_end}
{phang2}{cmd:. matrix L = cholesky(V)}{p_end}
{phang2}{cmd:. mhsmnorm logpost b 3, cholesky(L)}{p_end}


{title:Comment}

{pstd}
Other programs can be used to update single parameters. When parameters can
take any real value, use {helpb mhsnorm}; for parameters that must be
positive, use {helpb mhslogn}; and for parameters defined of any range (a,b),
where a and b may be finite or infinite, use {helpb mhstrnc}. 


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
