{smcl}
{* *! version 1.0.0  dec2012}{...}
{cmd:help mhsnorm} 
{hline}

{title:Title}

{p2colset 5 16 18 2}{...}
{p2col :{hi:mhsnorm} {hline 2}}Metropolis-Hastings algorithm based on a random walk with a Gaussian proposal distribution{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:mhsnorm} {it:logpost rowvector parameter} {ifin} {weight}{cmd:,}
{opt sd(#)} [{opt log(#)}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} below 
for more details on how to write the program for the log posterior.

{phang}
{it:rowvector} must be the name of the row vector containing the parameter
values.

{phang}
{it:parameter} must be the number of the parameter that is being updated.

{synoptset 20 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opt sd:(#)}} standard deviation of the proposal distribution{p_end}
{synopt:{opt logp:(#)}} value of log posterior at current parameter values{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:sd()} is required.{p_end}
{p 4 6 2}
{opt fweight}s are allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:mhsnorm} uses the Metropolis-Hastings algorithm to create a single update
of a single parameter using a Gaussian (normal) proposal distribution. Because
the proposals may take any value, this method is suited to parameters that are
defined between plus and minus infinity. Repeated calls to {cmd:mhsnorm} can
be used to create an MCMC algorithm and hence estimate the posterior
distribution of a set of parameters.  The user must supply a program that
evaluates the log posterior; this program must have the structure described
below. Parameters are passed in a row vector. The efficiency but not the
validity of the algorithm is affected by the choice of standard deviation of
the proposal distribution. Values between one-half and three-quarters of the
effective range of the posterior usually work well. It is not valid to use
previous results from the same chain to alter the standard deviation of the
proposal distribution.{p_end}


{title:Options}

{phang}
{opt sd(#)} specifies the standard deviation of the proposal distribution as
chosen by the user. This {it:#} affects the efficiency of the algorithm.
Values that are too small will lead to small changes in the parameter and thus
to slow convergence. Values that are too large produce proposals that are
frequently rejected, again causing slow convergence.{p_end}

{phang}
{opt logp(#)} specifies the value of the log posterior at the current point.
If a nonmissing value is given, then this value is used; otherwise, the value
is calculated using the user's program {it:logpost}. Specifying the value
when it is known will save one call to the user's program and so speed the
algorithm.{p_end}


{title:Stored results}

{pstd}
{cmd:mhsnorm} stores the following in {cmd:r()}:

{p2colset 9 20 22 2}{...}
{p2col 5 12 16 2: Scalars}{p_end}
{synopt:{cmd:r(logp)}}log posterior for the updated set of parameters{p_end}
{synopt:{cmd:r(accept)}}{cmd:1} if the proposal was accepted and {cmd:0} otherwise{p_end}
{p2colreset}{...}


{marker userprog}{...}
{title:Structure for the user's program}

{pstd}
The general form of the program must be{p_end}

        {cmd}program{txt} {it:logpost}
        {cmd}syntax anything [fweight] [if] [in]

        marksample touse

        tokenize `"`anything'"'
        local logp "`1'"
        local b    "`2'"
        local ipar = "`3'"
    
        local alpha = `b'[1,1]
        local beta  = `b'[1,2]{txt}
        ....
        {it: calculation of the log of the posterior}
        ....
        {cmd:end}

{pstd}If frequency weights and {cmd:if} and {cmd:in} conditions are not
required, then this can be simplified to

        {cmd:program} {it:logpost}
        {cmd}args logp b ipar
    
        local alpha = `b'[1,1]
        local beta  = `b'[1,2]{txt}
        ....
        {it:calculation of the log of the posterior}
        ....
        {cmd:end}

{pstd}
where {cmd:logp} is a scalar that will eventually contain the calculated value
of the log posterior, {cmd:b} is the row vector of parameters, and {cmd:ipar}
is the number of the parameter being updated.{p_end}

{pstd}
The calculation must return the value of the log posterior, although constants
in the summation that produces this value can be omitted.  Thus only terms
that depend on the parameter being updated need to be included.  Some gain in
efficiency is often possible by altering the calculation depending on which
parameter is being updated. For instance,{p_end}

        {cmd:program} {it:logpost}
        {cmd}args logp b ipar

        local alpha = `b'[1,1]
        local beta  = `b'[1,2]{txt}
        ....
        {cmd}if `ipar' == 1 { {txt}
        {it: calculation of the log of the conditional posterior for parameter 1}
        {cmd} }
        else{txt} ...
        ....
        {cmd:end}


{title:Example}

{phang}	{cmd:. mhsnorm logpost b 3, sd(0.1)}{p_end}


{title:Comment}

{pstd}
Gaussian proposals are useful for parameters that can take any value.  Use
{helpb mhslogn} for parameters that must be positive and {helpb mhstrnc} for
parameters defined over the range (a,b), where a and b may be finite or
infinite. {helpb mhsmnorm} simultaneously updates a set of parameters using a
multivariate Gaussian distribution.{p_end}


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
