*------------------------------
* Code for Chapter 11
* working directory must include the data file: data/larynx.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*==================================================================
* Case Study 12: Modelling Cancer Incidence
*==================================================================

*-------------------------------
* Polish men by apcfit
*-------------------------------
use data/larynx.dta, clear
apcfit if centre == "Poland" & gender == "male" , ///
   age(a) period(p) cohort(c) cases(cases) pop(pyrs)  ///
   agefit(afit) cohfit(cfit) perfit(pfit) nper(100000) refc(7)
twoway (rarea afit_lci afit_uci a, col(gs12) lcol(gs12) ) ///
   (line afit a, lpat(solid) lcol(black)) , leg(off) ///
    xtitle(Age) ytitle("Rate per 100,000") xlabel(none) nodraw
graph save temp/plot1.gph, replace
sort c
gen pc = p + 15
twoway (rarea pfit_lci pfit_uci pc, col(gs12) lcol(gs12) ) ///
 (line pfit pc, lpat(solid) lcol(black)) ///
 (rarea cfit_lci cfit_uci c, col(gs12) lcol(gs12) ) ///
 (line cfit c, lpat(solid) lcol(black)), leg(off) ///
 xtitle("        Cohort                          Period") ///
 ytitle(Rate Ratio) xlabel(none) nodraw
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph 

* FIGURE 11.1  
graph export eps\chap15_mark_Poland_male.eps, replace

pause "Press q to continue"

*---------------------------------------------------------
* WinBUGS ANALYSIS OF CANCER OF THE data/larynx IN POLISH MEN
*---------------------------------------------------------
tomodel chapter11a.do temp/model_apc.txt
/* 
model
{   
   for (i in 1:I) { 
     cases[i] ~ dpois(mu[i])
     log(mu[i]) <- log(pyr[i]) + alpha[age[i]] + betac[period[i]] +
                 gammac[cohort[i]]
   } 

# age effects
   alphamean[1]    <- 0.0
   alphaprec[1]    <- taua*1.0E-6
   alphamean[2]    <- 0.0  
   alphaprec[2]    <- taua*1.0E-6 
   for (a in 3:A){
      alphamean[a]    <- 2*alpha[a-1] - alpha[a-2]
      alphaprec[a]    <- taua 
   }

   taua ~ dgamma(1.0E-3,1.0E-3)

   for (a in 1:A){
      alpha[a]~ dnorm(alphamean[a],alphaprec[a])
   }

# period effects 
   betamean[1]    <- 0.0 
   betaprec[1]    <- taup*1.0E-6 
   betamean[2]    <- 0.0  
   betaprec[2]    <- taup*1.0E-6 
   for (p in 3:P) {
      betamean[p]    <- 2*beta[p-1] - beta[p-2]
      betaprec[p]    <- taup 
   }

   for (p in 1:P) {
      beta[p]~ dnorm(betamean[p],betaprec[p])
   }

# constraint - mean = 0
   for (p in 1:P) {
      betam[p]<-beta[p]-mean(beta[1:P])
   }
# constraint - linear trend = 0
   for (p in 1:P) {
      ivec[p] <- p - (P+1)/2
      pivec[p] <- ivec[p] * betam[p]
      betac[p] <- betam[p] - ivec[p] * sum(pivec[])/(P*(P+1)*(P-1)/12)
   }

   taup ~ dgamma(1.0E-3,1.0E-3)

# cohort effects 
   gammamean[1]    <- 0.0 
   gammaprec[1]    <- tauc*1.0E-6 
   gammamean[2]    <- 0.0  
   gammaprec[2]    <- tauc*1.0E-6 
   for (c in 3:C) {
      gammamean[c]    <- 2*gamma[c-1] - gamma[c-2]
      gammaprec[c]    <- tauc 
   } 

# constraint: mean = 0
   for (c in 1:C){
      gamma[c]~ dnorm(gammamean[c],gammaprec[c])
      gammac[c]<-gamma[c]-mean(gamma[1:C])
   }

   tauc~ dgamma(1.0E-3,1.0E-3)
}
*/
use data/larynx.dta, clear
keep if centre == "Poland" & gender == "male"
gen pyr = pyrs/100000
rename a age
rename p period
rename c cohort
count 
local I = r(N)
summarize age 
local A = r(max)
summarize period 
local P = r(max)
summarize cohort 
local C = r(max)
* DATA FILE
wbslist (vector cases pyr age period cohort , ///
  format(%5.0f %8.3f %3.0f %3.0f %3.0f) ) ///
  (I=`I',A=`A',P=`P',C=`C') using temp/data.txt , replace
* INITS FILE
wbslist (alpha=c(`A'{0}),beta=c(`P'{0}), ///
  gamma=c(`C'{0}),taua=100,tauc=100,taup=100) using temp/inits.txt, replace
* RUN OPENBUGS  
wbsscript using temp/script.txt , replace ///
  data(data.txt) model(model_apc.txt) init(inits.txt) ///
  burnin(1000) update(10000) log(logm.txt) path(`c(pwd)'/temp) ///
  set(alpha betac gammac taua tauc taup) coda(braym) openbugs 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
type temp/logm.txt
wbscoda using temp/braym, clear openbugs
save temp/braymcmcm.dta, replace


* SUMMARIZE & TRACE PLOT
mcmcstats *, save(estimates.dta,replace)
mcmctrace alpha_1 betac_4 gammac_6 gammac_13 , cgopt(title(No good: early drift) )

pause "Press q to continue"

*---------------------------------------------------------
* Burn-in clearly not long enough .. should have thought more
* carefully about the temp/initial values .. re-run
* starting from the previous solution
*---------------------------------------------------------
use data/larynx.dta, clear
keep if centre == "Poland" & gender == "male"
gen pyr = pyrs/100000
rename a age
rename p period
rename c cohort
count 
local I = r(N)
summarize age 
local A = r(max)
summarize period 
local P = r(max)
summarize cohort 
local C = r(max)
*----------------------------------------------------------
* read previous estimates and use them as starting values
*----------------------------------------------------------
use estimates.dta, clear
wbslist (alpha=c(`A'{mean[1]}),beta=c(`P'{mean[`A'+1]}),gamma=c(`C'{mean[`A'+`P'+1]}), ///
  taua={mean[`A'+`P'+`C'+1]},tauc={mean[`A'+`P'+`C'+2]},taup={mean[`A'+`P'+`C'+3]}) ///
  using temp/inits.txt, replace
wbsscript using temp/script.txt , replace ///
  data(data.txt) model(model_apc.txt) init(inits.txt) ///
  burnin(5000) update(10000) log(logm.txt) path(`c(pwd)'/temp) ///
  set(alpha betac gammac taua tauc taup) coda(braym) openbugs 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
type temp/logm.txt
wbscoda using temp/braym , clear openbugs
save temp/braymcmcm.dta, replace
*---------------------------------------------------------
* Re-check Convergence 
*---------------------------------------------------------
use temp/braymcmcm.dta, clear
mcmcstats * , save(temp/estimates.dta,replace)
mcmctrace alpha_1 betac_4 gammac_6 gammac_13 , cgopt( title(No early drift but poor mixing for alpha and gamma) )

pause "Press q to continue"

mcmcac alpha_1 betac_4 gammac_6 gammac_13 , cgopt(title(Autocorrelations very high))

pause "Press q to continue"


*---------------------------------------------------------
* Strong autocorrelation evident from trace & ac plots
*   .. re-run with longer chain and thinning 
*   run time about 4 minutes
*---------------------------------------------------------
wbsscript using temp/script.txt , replace ///
  data(data.txt) model(model_apc.txt) init(inits.txt) ///
  burnin(5000) update(200000) thin(20) log(logm.txt) dic path(`c(pwd)'/temp) ///
  set(alpha betac gammac taua taup tauc) coda(braym) openbugs 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
type temp/logm.txt
wbscoda using temp/braym , clear openbugs
save temp/braymcmcm.dta, replace
*---------------------------------------------------------
* Re-check Convergence 
*---------------------------------------------------------
use temp/braymcmcm.dta, clear
mcmcstats *, save(estimates.dta,replace)
mcmctrace alpha_1 betac_4 gammac_6 gammac_13 
pause "Press q to continue"
*---------------------------------------------------------
* Sample of cusum plots
*---------------------------------------------------------
mcmccusum alpha_1 betac_4 gammac_6 gammac_13 , ref(3)  cgopt(title(long run with thinning))
pause "Press q to continue"
*---------------------------------------------------------
* Sample of ac plots
*---------------------------------------------------------
mcmcac alpha_1 betac_4 gammac_6 gammac_13 , cgopt(title(long run with thinning))
pause "Press q to continue"

*---------------------------------------------------------
* Alpha: all and then first 6
*---------------------------------------------------------
mcmcmahal alpha* , distance(Da) cgopt(title(all alpha's)) replace
mcmcgeweke Da
pause "Press q to continue"
mcmcsection alpha_1 alpha_2 alpha_3 alpha_4 alpha_5 alpha_6
pause "Press q to continue"

*---------------------------------------------------------
* Beta: all and then first 6 
*---------------------------------------------------------
mcmcmahal beta* , distance(Db) cgopt(title(all beta's)) replace
mcmcgeweke Db
pause "Press q to continue"
mcmcsection betac_1 betac_2 betac_3 betac_4 betac_5 betac_6
pause "Press q to continue"

*---------------------------------------------------------
* Gamma: all and then first 6 
*---------------------------------------------------------
mcmcmahal gamma* , distance(Dg) cgopt(title(all gamma's)) replace
mcmcgeweke Dg
pause "Press q to continue"
mcmcsection gammac_1 gammac_2 gammac_3 gammac_4 gammac_5 gammac_6
pause "Press q to continue"

*---------------------------------------------------------
* Probably OK ... accept it 
* exponentiate the coefficients for the plot
*---------------------------------------------------------
use temp/braymcmcm.dta, clear
foreach x of varlist alp* beta* gam* {
  qui replace `x' = exp(`x')
}  
mcmcstats alp* gamma* bet*, save(temp/estimatesm.dta,replace)
*---------------------------------------------------------
* Plot in the style of apcfit
*---------------------------------------------------------
use temp/estimatesm.dta, clear
gen t = _n
twoway (rarea lb ub t if t < 10, col(gs12) lcol(gs12) ) ///
  (line mean t if t < 10, lpat(solid) lcol(black)) , ///
   xlabel(none) leg(off) xtitle(Age) ytitle("Rate per 100,000") nodraw
graph save temp/plot1.gph, replace
twoway (rarea lb ub t if inrange(t,10,23), col(gs12) lcol(gs12) ) ///
 (line mean t if inrange(t,10,23), lpat(solid) lcol(black)) ///
 (rarea lb ub t if t > 23, col(gs12) lcol(gs12) ) ///
 (line mean t if t >23, lpat(solid) lcol(black)) ///
 , leg(off) ytitle(Rate Ratio) xlabel(none) ///
 xtitle("        Cohort                          Period") nodraw
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph 
* FIGURE 11.2  
graph export eps\chap15_bray_Poland_male.eps, replace

pause "Press q to continue"

*---------------------------------
* Similar model in women
*---------------------------------
use data/larynx.dta, clear
keep if centre == "Poland" & gender == "female"
gen pyr = pyrs/100000
rename a age
rename p period
rename c cohort
count 
local I = r(N)
summarize age 
local A = r(max)
summarize period 
local P = r(max)
summarize cohort 
local C = r(max)
wbslist (alpha=c(`A'{0}),beta=c(`P'{0}),gamma=c(`C'{0}),taua=100,tauc=100,taup=100) ///
  using temp/inits.txt, replace
wbslist (vector cases pyr age period cohort , ///
  format(%5.0f %8.3f %3.0f %3.0f %3.0f) ) (I=`I',A=`A',P=`P',C=`C') ///
  using temp/data.txt, replace
wbsscript using temp/script.txt , replace ///
  data(data.txt) model(model_apc.txt) init(inits.txt) ///
  burnin(20000) update(200000) thin(20) log(logf.txt) dic path(`c(pwd)'/temp) ///
  set(alpha betac gammac taua taup tauc) coda(brayf) openbugs 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
type temp/logf.txt
wbscoda using `c(pwd)'/temp/brayf , clear openbugs
save temp/braymcmcf.dta, replace
use temp/braymcmcf.dta, clear
foreach x of varlist alp* beta* gam*{
  qui replace `x' = exp(`x')
}  
mcmcstats alp* gamma* bet*, save(temp/estimatesf.dta,replace)
use temp/estimatesf.dta, clear
gen t = _n
twoway (rarea lb ub t if t < 10, col(gs12) lcol(gs12) ) ///
  (line mean t if t < 10, lpat(solid) lcol(black)) , ///
   xlabel(none) leg(off) xtitle(Age) ytitle("Rate per 100,000") nodraw
graph save temp/plot1.gph, replace
twoway (rarea lb ub t if inrange(t,10,23), col(gs12) lcol(gs12) ) ///
 (line mean t if inrange(t,10,23), lpat(solid) lcol(black)) ///
 (rarea lb ub t if t > 23, col(gs12) lcol(gs12) ) ///
 (line mean t if t >23, lpat(solid) lcol(black)) ///
 , leg(off) ytitle(Rate Ratio) xlabel(none) ///
 xtitle("        Cohort                          Period") nodraw
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph 
* FIGURE 11.2  
graph export eps\chap15_bray_Poland_female.eps, replace

pause "Press q to continue"

*---------------------------------
* Combined model for men and women
*---------------------------------
tomodel chapter11a.do temp/model_apc_sex.txt
/*
model
{   
   for (i in 1:I) { 
     cases[i] ~ dpois(mu[i])
     log(mu[i]) <- log(pyr[i]) + alpha[age[i]] + betac[period[i]] + gammac[cohort[i]] + theta*sex[i]
   } 

   theta ~ dnorm(0,1.0E-6)
   
# age effects
   alphamean[1]    <- 0.0
   alphaprec[1]    <- taua*1.0E-6
   alphamean[2]    <- 0.0  
   alphaprec[2]    <- taua*1.0E-6 
   for (a in 3:A){
      alphamean[a]    <- 2*alpha[a-1] - alpha[a-2]
      alphaprec[a]    <- taua 
   }

   taua ~ dgamma(1.0E-3,1.0E-3)
   
   for (a in 1:A){
      alpha[a]~ dnorm(alphamean[a],alphaprec[a])
   }

# period effects 
   betamean[1]    <- 0.0 
   betaprec[1]    <- taup*1.0E-6 
   betamean[2]    <- 0.0  
   betaprec[2]    <- taup*1.0E-6 
   for (p in 3:P){
      betamean[p]    <- 2*beta[p-1] - beta[p-2]
      betaprec[p]    <- taup 
   }

   for (p in 1:P) {
      beta[p]~ dnorm(betamean[p],betaprec[p])
   }

# constraint - mean = 0
   for (p in 1:P) {
      betam[p]<-beta[p]-mean(beta[1:P])
   }
# constraint - linear trend = 0
   for (p in 1:P) {
      ivec[p] <- p - (P+1)/2
      pivec[p] <- ivec[p] * betam[p]
      betac[p] <- betam[p] - ivec[p] * sum(pivec[])/(P*(P+1)*(P-1)/12)
   }

   taup ~ dgamma(1.0E-3,1.0E-3)

# cohort effects 
   gammamean[1]    <- 0.0 
   gammaprec[1]    <- tauc*1.0E-6 
   gammamean[2]    <- 0.0  
   gammaprec[2]    <- tauc*1.0E-6 
   for (c in 3:C){
      gammamean[c]    <- 2*gamma[c-1] - gamma[c-2]
      gammaprec[c]    <- tauc 
   } 

# constraint: mean = 0
   for (c in 1:C){
      gamma[c]~ dnorm(gammamean[c],gammaprec[c])
      gammac[c]<-gamma[c]-mean(gamma[1:C])
   }

   tauc~ dgamma(1.0E-3,1.0E-3)
}
*/
use data/larynx.dta, clear
keep if centre == "Poland" 
gen pyr = pyrs/100000
rename a age
rename p period
rename c cohort
gen sex = gender == "female"
count 
local I = r(N)
summarize age 
local A = r(max)
summarize period 
local P = r(max)
summarize cohort 
local C = r(max)
wbslist (alpha=c(`A'{0}),beta=c(`P'{0}),gamma=c(`C'{0}),theta=0,taua=100,tauc=100,taup=100) ///
  using temp/inits.txt, replace
wbslist (vector cases pyr age period cohort sex, ///
  format(%5.0f %8.3f %3.0f %3.0f %3.0f %3.0f) ) (I=`I',A=`A',P=`P',C=`C') ///
  using temp/data.txt, replace
wbsscript using temp/script.txt , replace ///
  data(data.txt) model(model_apc_sex.txt) init(inits.txt) ///
  burnin(20000) update(200000) thin(20) log(logmf.txt) dic path(`c(pwd)'/temp) ///
  set(alpha betac gammac theta taua taup tauc) coda(braymf) openbugs 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
type temp/logmf.txt
wbscoda using `c(pwd)'/temp/braymf , clear openbugs
save temp/braymcmcmf.dta, replace

pause "Press q to continue"
