*------------------------------
* Code for Chapter 11
* working directory must include the data file: asthma.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*==================================================================
* Case Study 12: recurrent asthma attacks
*==================================================================

*-------------------------------------------
* Plot of histories of 20 children
*-------------------------------------------
use data/asthma.dta, clear
bys id: gen end = start[_n+1]
twoway (rcap start stop id if id <= 20, horiz)                ///
   (rcap stop end id if id <= 20 & event == 1 & id[_n+1]==id, ///
     horiz lwidth(thick) lcol(black))                         ///
   (rcap stop stop id if id <= 20 & event == 1 & end==.,      ///
     horiz lwidth(thick) lcol(black)),                        ///
   leg(off) xtitle(Calendar time (days)) ytitle(Child)        ///
   xlabel(0(100)700) xscale(range(0 700))
* FIGURE 11.11  
graph export eps\duch_pattern.eps, replace
pause "Press q to continue"


*-------------------------------------------
* Events eby calandar
*-------------------------------------------
forvalues i=1/12 {
   count if event == 1 & stop > (`i'-1)*50 & stop < `i'*50
}
pause "Press q to continue"

*-------------------------------------------
* Events by gap time
*-------------------------------------------
* DOF: 
gen gap = stop - start
forvalues i=1/10 {
  qui count if gap > (`i'-1)*50 
  local a = r(N)
  qui count if event == 1 & gap > (`i'-1)*50 & gap < `i'*50
  display %4.0f r(N) %5.2f r(N)/`a' 
}
pause "Press q to continue"

*-------------------------------------------
* Hazard plots
*-------------------------------------------
* DOF: 
stset stop , id(id) failure(event) exit(time .) time0(start)
sts graph , hazard yscale(range(0 0.01)) ///
   ylabel(0(0.002)0.01) title(Calendar time)
graph save temp/plot1.gph, replace
stset stop , failure(event) origin(start) 
sts graph , hazard yscale(range(0 0.01)) ///
   ylabel(0(0.002)0.01) title(Gap time)
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 11.12  
graph export eps\chap15_hazard.eps, replace
pause "Press q to continue"

*-------------------------------------------
* Events by gap time
*-------------------------------------------
tomodel chapter11d.do temp/modelgap.txt
/* 
model {
   for (i in 1:1037) {
      log(lambda[i]) <- alpha + beta*trt[i] + u[id[i]]
      t[i] ~ dweib(v,lambda[i]) C(tcen[i],) 
   }
   for (j in 1:232) {
      u[j] ~ dnorm(0,tau)
   }
   alpha ~ dnorm(-4.5,1)
   beta ~ dnorm(0,0.4)
   tau ~ dgamma(2,0.5)
   v ~ dgamma(5,5)
}      
*/
use data/asthma.dta, clear
gen t = stop - start
gen tcen = (event==0)*t
replace t = . if event == 0
wbslist (vector id trt t tcen, format(%5.0f)) using temp/data.txt, replace
wbslist (alpha=-4.5,beta=0,tau=4,v=1,u=c(232{0.5*rnormal()})) ///
  using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(modelgap.txt) init(inits.txt) ///
  burnin(2000) update(20000) thin(5) dic logfile(log.txt) ///
  set(alpha beta tau v) coda(duchgap) openbugs path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/duchgap, clear openbugs
save temp/duchgapmcmc.dta, replace
mcmcstats alpha beta tau v
mcmctrace alpha beta tau v
pause "Press q to continue"
mcmcstats alpha beta tau v
pause "Press q to continue"

*-------------------------------------------
* Gap time fitted hazard
*-------------------------------------------
use temp/duchgapmcmc.dta, clear
foreach t of numlist 1 50(50)500 {
   gen h0_`t' = exp(alpha)*v*`t'^(v-1) 
   gen h1_`t' = exp(alpha+beta)*v*`t'^(v-1) 
}
/*
mcmcstats h*, save(duch_haz.dta,replace)
*/
mcmcstats h*, save(temp/duch_haz.dta,replace)
pause "Press q to continue"
use temp/duch_haz.dta, clear
egen trt = seq(), t(2)
egen t = seq(), b(2) t(11)
replace t = (t-1)*50 + (t==1)
twoway (line ub t if trt == 1, lpat(dash) lcol(blue)) ///
    (line mean t if trt == 1, lpat(solid) lcol(blue)) ///
    (line lb t if trt == 1, lpat(dash) lcol(blue)) ///
   (line ub t if trt == 2, lpat(dash)  lcol(red))  ///
    (line mean t if trt == 2, lpat(solid) lcol(red)) ///
    (line lb t if trt == 2, lpat(dash) lcol(red)), ///
   leg(off)
pause "Press q to continue"

*-------------------------------------------
* Calendar time model
*-------------------------------------------
tomodel chapter11d.do temp/modelcal.txt
/* 
 model {
   for (i in 1:1037) {
      log(lambda[i]) <- alpha + beta*trt[i] + u[id[i]]
      logL[i] <- event[i]*(log(lambda[i])+log(v)+(v-1)*log(t[i]))-lambda[i]*(pow(t[i],v)-pow(tleft[i],v))
      zero[i] ~ dloglik(logL[i])   
   }
   for (j in 1:232) {
      u[j] ~ dnorm(0,tau)
   }
   alpha ~ dnorm(-4.5,1)
   beta ~ dnorm(0,0.4)
   tau ~ dgamma(2,0.5)
   v ~ dgamma(5,5)
}      
*/
use data/asthma.dta, clear
rename stop t 
rename start tleft
gen zero = 0
wbslist (vector id trt tleft t event zero, format(%5.0f)) ///
  using temp/data.txt, replace
wbslist (alpha=-4.5,beta=0,tau=4,v=1,u=c(232{0.5*rnormal()})) ///
  using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(modelcal.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) dic logfile(log.txt) ///
  set(alpha beta tau v) coda(duchcal) openbugs path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/duchcal, clear openbugs
save temp/duchcalmcmc.dta, replace
mcmcstats alpha beta tau v
mcmctrace alpha beta tau v
pause "Press q to continue"

*-------------------------------------------
* Calendar time fitted hazard
*-------------------------------------------
use temp/duchcalmcmc.dta, clear
foreach t of numlist 1 50(50)500 {
   gen h0_`t' = exp(alpha)*v*`t'^(v-1) 
   gen h1_`t' = exp(alpha+beta)*v*`t'^(v-1) 
}
mcmcstats h*, save(temp/duch_haz.dta,replace)
use temp/duch_haz.dta, clear
egen trt = seq(), t(2)
egen t = seq(), b(2) t(11)
replace t = (t-1)*50 + (t==1)
twoway (line ub t if trt == 1, lpat(dash) lcol(blue)) ///
    (line mean t if trt == 1, lpat(solid) lcol(blue)) ///
    (line lb t if trt == 1, lpat(dash) lcol(blue)) ///
   (line ub t if trt == 2, lpat(dash)  lcol(red))  ///
    (line mean t if trt == 2, lpat(solid) lcol(red)) ///
    (line lb t if trt == 2, lpat(dash) lcol(red)), ///
   leg(off)
pause "Press q to continue"

   
