*------------------------------
* Code for Chapter 5
* working directory must include the data file: betablocker.dta
*------------------------------
clear
set scheme sj
program drop _all
pause off

*-------------------------------
* Section 5.2
* No early drift - no autocorrelation
* figure 5.1
*-------------------------------
set seed 376015
set obs 1000
local mu = 5 
local sigma = 1
local rho = 0
local f = sqrt(1-`rho'*`rho')*`sigma'
gen     theta = rnormal(`mu',`sigma') in 1
replace theta = rnormal(`mu'*(1-`rho') + `rho'*theta[_n-1],`f') in 2/1000

mcmctrace theta   
graph save temp/plot1.gph, replace
mcmccusum theta , reference(1)
graph save temp/plot2.gph, replace
* FIGURE 5.1  
graph combine temp/plot1.gph temp/plot2.gph
graph export eps/chap5_1.eps, replace

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 5.2
* No Early drift - Autocorrelation
* figure 5.2
*-------------------------------
clear
set seed 376015
set obs 1000
local mu = 5 
local sigma = 1
local rho = 0.8
local f = sqrt(1-`rho'*`rho')*`sigma'
gen     theta = rnormal(`mu',`sigma') in 1
replace theta = rnormal(`mu'*(1-`rho') + `rho'*theta[_n-1],`f') in 2/1000

mcmctrace theta   
graph save temp/plot1.gph, replace
mcmccusum theta , reference(1)
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 5.2  
graph export eps/chap5_2.eps, replace

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 5.2
* Early drift no autocorrelation
* figure 5.3
*-------------------------------
clear
set seed 376015
set obs 1000
local mu = 5 
local sigma = 1
local rho = 0
local f = sqrt(1-`rho'*`rho')*`sigma'
gen     theta = rnormal(`mu',`sigma') in 1
replace theta = rnormal(`mu'*(1-`rho') + `rho'*theta[_n-1],`f') in 2/1000
replace theta = theta - 1 + _n/200 in 1/200

mcmctrace theta   
graph save temp/plot1.gph, replace
mcmccusum theta , reference(1)
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 5.3  
graph export eps/chap5_3.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.2
* Early drift & autocorrelation
* figure 5.4
*-------------------------------
clear
set seed 376015
set obs 1000
local mu = 5 
local sigma = 1
local rho = 0.8
local f = sqrt(1-`rho'*`rho')*`sigma'
gen     theta = rnormal(`mu',`sigma') in 1
replace theta = rnormal(`mu'*(1-`rho') + `rho'*theta[_n-1],`f') in 2/1000
replace theta = theta - 1 + _n/200 in 1/200

mcmctrace theta   
graph save temp/plot1.gph, replace
mcmccusum theta , reference(1)
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 5.4  
graph export eps/chap5_4.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.2
* Early drift
* Geweke test
*-------------------------------

mcmcgeweke theta

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.3
* Convergence
* figure 5.5
*-------------------------------

clear
program drop _all
set obs 5000
set seed 376015
local mu = 5 
local sigma = 1
local rho = 0.8
local f = sqrt(1-`rho'*`rho')*`sigma'
qui gen     theta = rnormal(`mu'-1,`sigma') in 1
qui replace theta = rnormal(`mu'*(1-`rho') + `rho'*theta[_n-1],`f') in 2/5000

mcmcsection theta in 1/500
graph save temp/plot1.gph, replace
mcmcsection theta
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 5.5  
graph export eps/chap5_5.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.3
* length of run
*-------------------------------
mcmclength theta , target(0.025)

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.3
* AC
* figure 5.6
*-------------------------------

* FIGURE 5.6  
mcmcac theta 
graph export eps/chap5_ac.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.4
* intervals
* figure 5.7
*-------------------------------

clear
set obs 1000
set seed 376015
local rho = 0.8
local mu = 5 
local sigma = 1
local f = sqrt(1-`rho'*`rho')*`sigma'
qui gen     theta1 = rnormal(`mu',`sigma') in 1
qui replace theta1 = rnormal(`mu'*(1-`rho') + `rho'*theta1[_n-1],`f') in 2/1000
qui gen     theta2 = rnormal(`mu',`sigma') in 1
qui replace theta2 = rnormal(`mu'*(1-`rho') + `rho'*theta2[_n-1],`f') in 2/1000
qui replace theta2 = theta2 - 3 + 3*_n/250  in 1/250
qui gen     theta3 = rnormal(`mu',`sigma') in 1
qui replace theta3 = rnormal(`mu'*(1-`rho') + `rho'*theta3[_n-1]) in 2/1000
qui replace theta3 = theta3 + 3 - 3*_n/500  in 1/500
gen Iteration = _n
stack Iteration theta1 Iteration theta2 Iteration theta3, into(Iteration theta) clear
rename _stack chain

* FIGURE 5.7  
mcmcintervals theta, chain(chain)
graph export eps/chap5_7.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.4
* bgr
* figure 5.8
*-------------------------------

* FIGURE 5.8  
mcmcbgr theta, chain(chain) i(Iteration)
graph export eps/chap5_6.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.6
* Beta-blockers priors
* figure 5.9
*-------------------------------

twoway function y=betaden(4,36,x) , range(0 0.4) xtitle(Mortality in the placebo group) yscale(off)
graph save temp/plot1.gph, replace
twoway function y=gammaden(2,0.5,0,x) , range(0 5) xtitle(Relative Risk) yscale(off)
graph save temp/plot2.gph, replace
* FIGURE 5.9  
graph combine temp/plot1.gph temp/plot2.gph
graph export eps/betaprior.eps, replace

pause "Press q to continue"
*================================================================================
*-------------------------------
* Section 5.6
* Beta-blockers model fit
*-------------------------------
program drop _all
use data/betablocker.dta, clear
set more off
set seed 386719
program logpost
  args logp b p

  local phi = `b'[1,23]
  if `p' == 23 {
     scalar `logp' = log(`phi') - 2*`phi'
     forvalues i=1/22 {
        local p_i = `b'[1,`i']
        scalar `logp' = `logp' + (35+n0[`i']-y0[`i'])*log(1-`p_i') +          ///
            (y0[`i']+3)*log(`p_i') + (n1[`i']-y1[`i'])*log(1-`phi'*`p_i')     ///
            + y1[`i']*log(`phi'*`p_i')
     }
  }
  else {
     local p_i = `b'[1,`p']
     scalar `logp' = (35+n0[`p']-y0[`p'])*log(1-`p_i') +                      ///
        (y0[`p']+3)*log(`p_i')  + (n1[`p']-y1[`p'])*log(1-`phi'*`p_i') +      ///
        y1[`p']*log(`phi'*`p_i')
  }
end

matrix theta = J(1,23,.1)
matrix theta[1,23] = 1
matrix sigma = J(1,23,0.1)
mcmcrun  logpost theta using temp/temp.csv,                               ///
    samp(22(mhstrnc , sd(sigma) lb(0) ub(1)) (mhslogn , sd(sigma)) ) ///
    burn(1000) adapt update(25000) savelogp thin(5)                  ///
    par(p1-p22 phi) replace dots(0)

import delimited temp/temp.csv, clear
save data/betablocker_mcmc.dta, replace
* FIGURE 5.10  
mcmctrace logp
graph export eps/logptrace.eps, replace

* FIGURE 5.11  
mcmcmahal p1-p22
graph export eps/mahalblocker.eps, replace

pause "Press q to continue"
*================================================================================
