*------------------------------
* Code for Chapter 9
* working directory must include the data file: salm.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*------------------------------------
* 9.3.1 Fitting the salmonella model
*------------------------------------
tomodel chapter9.do temp/model.txt V1
/* 
model {
  for (i in 1 : 18) {
     # MODEL FITTING
     y[i] ~ dpois(mu[i])
     log(mu[i]) <- beta0 + beta1*x1[i] + beta2*x2[i] + u[i]
     u[i] ~ dnorm(0.0, tau)
     # PREDICTIONS
     v[i] ~ dnorm(0.0, tau)   
     log(m[i]) <- beta0 + beta1*x1[i] + beta2*x2[i] + v[i]
     ystar[i] ~ dpois(m[i])
   }
   beta0 ~ dnorm(0.0,1.0E-6)
   beta1 ~ dnorm(0.0,1.0E-6)
   beta2 ~ dnorm(0.0,1.0E-6)
   tau ~ dgamma(1.0E-3, 1.0E-3)
}
*/
use data/salm.dta, clear
gen x1 = log(x+10)
gen x2 = x/100
save temp/salmplus.dta, replace
summarize x1
local m1 = r(mean)
replace x1 = x1 - r(mean)
summarize x2
local m2 = r(mean)
replace x2 = x2 - r(mean)
wbslist (vector x1 x2 y) using temp/data.txt, replace
wbslist (beta0=0,beta1=0,beta2=0,tau=1) using temp/inits.txt, replace
wbsscript using temp/script.txt, ///
   model(model.txt) data(data.txt) init(inits.txt) ///
   burn(1000) update(10000) set(beta0 beta1 beta2 tau u ystar) ///
   coda(sal) replace openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs time
wbscoda using temp/sal, clear openbugs
replace beta0 = beta0 - `m1'*beta1 - `m2'*beta2
gen sd = 1/sqrt(tau)

mcmcstats beta0 beta1 beta2 sd
save temp/mcmc.dta, replace
pause "Press q to continue"

*------------------------------------
* 9.3.1 Fitting the salmonella model
* clearly incorrect linear model
*------------------------------------
tomodel chapter9.do temp/model.txt V2
/* 
model {
  for (i in 1 : 18) {
     # MODEL FITTING
     y[i] ~ dpois(mu[i])
     log(mu[i]) <- beta0 + beta2*x2[i] + u[i]
     u[i] ~ dnorm(0.0, tau)
     # PREDICTIONS
     v[i] ~ dnorm(0.0, tau)   
     log(m[i]) <- beta0 +  beta2*x2[i] + v[i]
     ystar[i] ~ dpois(m[i])
   }
   beta0 ~ dnorm(0.0,1.0E-6)
   beta2 ~ dnorm(0.0,1.0E-6)
   tau ~ dgamma(1.0E-3, 1.0E-3)
}
*/
use temp/salmplus.dta, replace
summarize x2
local m2 = r(mean)
replace x2 = x2 - r(mean)
wbslist (vector x2 y) using temp/data.txt, replace
wbslist (beta0=0,beta2=0,tau=1) using temp/inits.txt, replace
wbsscript using temp/script.txt, ///
     model(model.txt) data(data.txt) init(inits.txt) ///
     burn(1000) update(10000) ///
     set(beta0 beta2 tau u ystar) coda(sal) ///
     replace openbugs path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/sal, clear openbugs
replace beta0 = beta0 - `m2'*beta2
gen sd = 1/sqrt(tau)
mcmcstats beta0 beta2 sd
save temp/mcmce.dta, replace

pause "Press q to continue"

*------------------------------------
* Histograms for the two models
*------------------------------------
mcmccheck, d(y[12]) p(ystar_12) df(data/salm.dta) pf(temp/mcmc.dta) ///
    gopt(title(model: log(x+10)+x/100) nodraw)
graph save temp/plot1.gph, replace
mcmccheck, d(y[12]) p(ystar_12) df(data/salm.dta) pf(temp/mcmce.dta) ///
   gopt(title(model: x/100) nodraw)
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 9.1
graph export eps/salmres12.eps, replace

pause "Press q to continue"

*------------------------------------
* Residual plots
*------------------------------------
mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmc.dta) ///
    x(fit) y(resid) gopt(title(model: log(x+10)+x/100)  nodraw)
graph save temp/plot1.gph, replace
mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmce.dta) ///
    x(fit) y(resid) gopt(title(model: x/100)  nodraw)
graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 9.2
graph export eps/salmresfit.eps, replace

pause "Press q to continue"
   
*------------------------------------
* fitted values
*------------------------------------
mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmc.dta) ///
    x(log(x+10)) y(fit) 
* FIGURE 9.3
graph export eps/salmfit.eps, replace

pause "Press q to continue"

*------------------------------------
* Saving the measures of surprise
*------------------------------------  
mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmc.dta) ///
   x(log(x+10)) y(res) save(temp/salmres.dta,replace) 
   
pause "Press q to continue"
   
*------------------------------------
* Labelled PPP
*------------------------------------  
use data/salm.dta, clear
gen id = _n
save temp/salmid.dta, replace
mcmccheck, d(y) p(ystar_) df(temp/salmid.dta) pf(temp/mcmce.dta) ///
    x(log(x+10)) y(ppp) plot(scatter) lv(id)
* FIGURE 9.4
graph export eps/salmppp.eps, replace

pause "Press q to continue"

*------------------------------------
* box plot
*------------------------------------  
mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmce.dta) ///
   y(rps) plot(box) gopt( over(x) b1title(Dose) ytitle(Relative Surprise) )  
* FIGURE 9.5
graph export eps/salmrpsdose.eps, replace

pause "Press q to continue"

*------------------------------------
* EPP
*------------------------------------  
mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmc.dta) ///
    yaxis(resid) plot(epp) nsamples(50) ///
    gopt(nodraw title(model: log(x+10)+x/100) yscale(range(-3 6)) ylabel(-2(2)6) ///
   xscale(range(-2 3)) xlabel(-2(1)3)  )
graph save temp/plot1.gph, replace

mcmccheck, d(y) p(ystar_) df(data/salm.dta) pf(temp/mcmce.dta) ///
yaxis(resid) plot(epp) nsamples(50) gopt(nodraw title(model: x/100) yscale(range(-3 6)) ylabel(-2(2)6) xscale(range(-2 3)) xlabel(-2(1)3)   )
graph save temp/plot2.gph, replace

graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 9.6
graph export eps/salmepp.eps, replace

pause "Press q to continue"

*----------------------------------------
* EPP TESTS
*----------------------------------------
*-------------------------------------------
* LOGNORMAL
*-------------------------------------------
clear
program drop _all
program logpost
   args logp b 
   
   local mu = `b'[1,1]
   local tau = `b'[1,2]
   local sd = 1/sqrt(`tau')
   scalar `logp' = 0
   logdensity "normal" `logp' y `mu' `sd'
   logdensity "normal" `logp' `mu' 1 1
   logdensity "gamma" `logp' `tau' 2 0.5
end

program ys, rclass
   args b
   
   local mu = `b'[1,1]
   local tau = `b'[1,2]
   local sd = 1/sqrt(`tau')
   tempname y
   matrix `y' = J(1,40,0)
   forvalues i=1/40 {
       matrix `y'[1,`i'] = rnormal(`mu',`sd')
   }
   return matrix pred = `y'
end

set obs 40
set seed 275904
gen y = exp(rnormal())
histogram y, bin(20) start(0)
graph save temp/plot1.gph, replace
save temp/logndata.dta, replace
matrix b = ( 1, 1 )
matrix s = (0.5, 0.5)
mcmcrun logpost b using temp/temp.csv, ///
   sampl( (mhsnorm, sd(s)) (mhslogn, sd(s)) ) ///
   burn(500) update(2000) par(mu tau) adapt ///
   pred(ys) jpost replace
import delimited temp/temp.csv, clear
gen sd = 1/sqrt(tau)
mcmcstats mu tau sd
save temp/lognmcmc.dta, replace
mcmccheck, d(y) p(ys) dfile(temp/logndata.dta) pfile(temp/lognmcmc.dta) ///
   plot(epp) nsam(50)
graph save temp/plot2.gph, replace  
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 9.7
graph export eps/chap9_epplogn.eps, replace
pause "Press q to continue"


*-------------------------------------------
* POISSON
*-------------------------------------------
clear
program drop _all
program logpost
   args logp b 
   
   local mu = `b'[1,1]
   scalar `logp' = 0
   logdensity "poisson" `logp' y `mu' 
   logdensity "gamma" `logp' `mu' 2 0.5
end

program ys, rclass
   args b
   
   local mu = `b'[1,1]
   tempname y
   matrix `y' = J(1,40,0)
   forvalues i=1/40 {
       matrix `y'[1,`i'] = rpoisson(`mu')
   }
   return matrix pred = `y'
end

set obs 40
set seed 275904
gen mu = rgamma(2,5/2)
gen y = rpoisson(mu)
histogram y,  discrete
graph save temp/plot1.gph, replace
save temp/poisdata.dta, replace
matrix b = ( 5 )
matrix s = (0.5)
mcmcrun logpost b using temp/temp.csv, ///
   sampl(  (mhslogn, sd(s)) ) ///
   burn(500) update(2000) par(mu) adapt ///
   pred(ys) jpost replace
import delimited temp/temp.csv, clear
mcmcstats mu 
save temp/poismcmc.dta, replace
mcmccheck, d(y) p(ys) dfile(temp/poisdata.dta) pfile(temp/poismcmc.dta) ///
   plot(epp) nsam(50)
graph save temp/plot2.gph, replace  
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 9.8
graph export eps/chap9_epppois.eps, replace
pause "Press q to continue"

*------------------------------------
* Summary plots
*------------------------------------  
mcmccheck, d(y) p(ystar_) df(data/salm.dta) ///
   pf(temp/mcmc.dta) plot(summary)
* FIGURE 9.9
graph export eps/salmsummary.eps, replace

mcmccheck, d(y) p(ystar_) df(data/salm.dta) ///
   pf(temp/mcmce.dta) plot(summary)   
* FIGURE 9.10
graph export eps/salmsummarye.eps, replace

pause "Press q to continue"

*------------------------------------
* Model checking with Stata
*------------------------------------
program drop _all 
use data/salm.dta, clear
gen x1 = log(x+10)
gen x2 = x/100
summarize x1
local m1 = r(mean)
replace x1 = x1 - r(mean)
summarize x2
local m2 = r(mean)
replace x2 = x2 - r(mean)

program logpost
   args logp b ipar

   local logtau = log(`b'[1,1])
   tempvar xb
   gen `xb' = `b'[1,2]+`b'[1,3]*x1+`b'[1,4]*x2
   scalar `logp' = 0
   forvalues i=1/18 {
      local j = `i' + 4
      scalar `logp' = `logp' + y[`i']*(`xb'[`i']+`b'[1,`j']) ///
         - exp(`xb'[`i']+`b'[1,`j']) ///
         - 0.5*`b'[1,`j']*`b'[1,`j']*`b'[1,1] + 0.5*`logtau'
   }
   scalar `logp' = `logp' - 0.5*`b'[1,2]*`b'[1,2]/1000000  ///
       - 0.5*`b'[1,3]*`b'[1,3]/1000000 - 0.5*`b'[1,4]*`b'[1,4]/1000000 ///
       - 0.999*`logtau' - `b'[1,1]/1000
end

program pd, rclass
   args b
   
   local sd = 1/sqrt(`b'[1,1])
   tempvar xb u
   gen `u' = rnormal(0,`sd')
   gen `xb' = exp(`b'[1,2]+`b'[1,3]*x1+`b'[1,4]*x2+`u')
   tempname rv
   matrix `rv' = J(1,18,0)
   forvalues i=1/18 {
      matrix `rv'[1,`i'] = rpoisson(`xb'[`i'])
   }
   return matrix pred = `rv'
end
   
matrix theta = J(1,22,0)
matrix theta[1,1] = 1
matrix s = J(1,22,0.5)
mcmcrun logpost theta using temp/temp.csv, ///
   samp( (mhslogn, sd(s)) 21(mhsnorm, sd(s)) ) ///
   burnin(1000) adapt update(10000)  ///
   par(tau beta0 beta1 beta2 u1-u18) savelogp ///
   predict(pd) jpost replace
import delimited temp/temp.csv, clear
replace beta0 = beta0 - `m1'*beta1 - `m2'*beta2
gen sd = 1/sqrt(tau)
save temp/mcmcs.dta, replace
mcmccheck, d(y) p(pd) df(data/salm.dta) pf(temp/mcmcs.dta) plot(summary)

pause "Press q to continue"

*------------------------------------
* Model checking with Mata
*------------------------------------  
program drop _all
use data/salm.dta, clear
gen x1 = log(x+10)
gen x2 = x/100
summarize x1
local m1 = r(mean)
replace x1 = x1 - r(mean)
summarize x2
local m2 = r(mean)
replace x2 = x2 - r(mean)
set matastrict off
mata:
mata clear
real scalar logpost(real matrix X,real rowvector t,real scalar p)
{
  sd = 1/sqrt(t[1])
  xb = exp(t[2]:+t[3]*X[,2]+t[4]*X[,3]+t[|5\22|]')
  logp = logdensity("poisson",X[,1],xb)+
         logdensity("normal",t[|5\22|],0,sd)+
       logdensity("normal",t[|2\4|],0,1000)+
       logdensity("gamma",t[1],0.001,1000)
  return(logp)
}
real rowvector pd(real matrix X,real rowvector t)
{
  sd = 1/sqrt(t[1])
  xb = exp(t[2]:+t[3]*X[,2]+t[4]*X[,3]:+rnormal(18,1,0,sd))
  return(rpoisson(1,1,xb'))
}

end

matrix theta = J(1,22,0)
matrix theta[1,1] = 1
matrix s = J(1,22,0.5)

mcmcrun logpost X theta using temp/temp.csv, ///
  samp( (mhslogn, sd(s)) 21(mhsnorm, sd(s)) ) ///
  burnin(1000) adapt update(10000) jpost dots(0) replace ///
  par(tau beta0 beta1 beta2 u1-u18) savelogp ///
  mata data(X=(y x1 x2) theta s) predict(pd) 

import delimited temp/temp.csv, clear
replace beta0 = beta0 - `m1'*beta1 - `m2'*beta2
gen sd = 1/sqrt(tau)
save temp/mcmcm.dta, replace
mcmccheck, d(y) p(pd) dfile(data/salm.dta) pfile(temp/mcmcm.dta) plot(summary) 

pause "Press q to continue"
